@@ -20,6 +20,7 @@ ADDITIONAL_PROPERTIES = "_additionalProperties"
 IS_IMMUTABLE = "_immutable"
 OPTIONAL_FIELDS = "_optional"
 MUST_BE_UNIQUE = "_must_be_unique"
+IGNORE_NONE_VALUES = "_ignore_none"
 MAX_NUMBER_OF_INSTANCES_TO_VERIFY_UNIQUENESS = 100000
 
 py_version = sys.version_info[0:2]
@@ -419,6 +420,7 @@ def _instantiate_fields_if_needed(cls_dict: dict, defaults: dict):
                 IS_IMMUTABLE,
                 DEFAULTS,
                 OPTIONAL_FIELDS,
+                IGNORE_NONE_VALUES,
             }
             and not isinstance(val, Field)
             and not key.startswith("__")
@@ -686,6 +688,11 @@ class Structure(UniqueMixin, metaclass=StructMeta):
                 # this raises an exception:
                 Foo(id = 1, a = 2)
 
+        _ignore_none(bool): optional
+             Ignore assignment to None for any field value.
+             Default is False.
+             Required fields never ignore None (since they are required)
+
     Decorating it with @unique ensures that no all instances of this structure will be unique. It
     will raise an exception otherwise (see "Uniqueness" section).
     """
@@ -717,6 +724,15 @@ class Structure(UniqueMixin, metaclass=StructMeta):
             if key in self.__dict__:
                 raise ValueError("Structure is immutable")
             value = deepcopy(value)
+        if all(
+            [
+                getattr(self, IGNORE_NONE_VALUES, False),
+                value is None,
+                key not in getattr(self.__class__, REQUIRED_FIELDS, []),
+            ]
+        ):
+            return
+
         super().__setattr__(key, value)
 
         if (
